#!/usr/bin/env python

import unittest

import math

from baker        import run_baker

from grid.DD      import grid
from grid.simplex import contains

def exact_func(X):
  x = X[0]
  y = X[0]
  return 1 - math.sin((x-0.5)**2 + (y-0.5)**2)

class TestSequenceFunctions(unittest.TestCase):
  def setUp(self):
    self.points = [
                        [ 0.25, 0.40], # 0
                        [ 0.60, 0.80], # 1
                        [ 0.65, 0.28], # 2
                        [ 0.28, 0.65], # 3
                        [ 1.00, 0.75], # 4
                        [ 0.30, 0.95], # 5
                        [ 0.80, 0.50], # 6
                        [ 0.35, 0.15], # 7
                      ]
    self.q = [exact_func(p) for p in self.points]

    self.X = [0.25, 0.4001]
    self.X = [0.55, 0.45]

    self.g = grid(self.points, self.q)
    self.g.construct_connectivity()
    self.R = self.g.create_mesh(range(3))

    self.exact = exact_func(self.X)


    self.accuracy = 8

  def test_R_contains_X(self):
    self.assertTrue(contains(self.X, self.R.points))

  def test_RunBaker_1_extra_point(self, extra=1):
    S = self.g.create_mesh(range(3, 3 + extra))
    answer = run_baker(self.X, self.R, S)
    lin_err   = abs(self.exact - answer['qlin'])
    final_err = abs(self.exact - answer['final'])
    self.assertTrue(lin_err >= final_err)
  def test_RunBaker_2_extra_point(self, extra=2):
    S = self.g.create_mesh(range(3, 3 + extra))
    answer = run_baker(self.X, self.R, S)
    lin_err   = abs(self.exact - answer['qlin'])
    final_err = abs(self.exact - answer['final'])
    self.assertTrue(lin_err >= final_err)
  def test_RunBaker_3_extra_point(self, extra=3):
    S = self.g.create_mesh(range(3, 3 + extra))
    answer = run_baker(self.X, self.R, S)
    lin_err   = abs(self.exact - answer['qlin'])
    final_err = abs(self.exact - answer['final'])
    self.assertTrue(lin_err >= final_err)
  def test_RunBaker_4_extra_point(self, extra=4):
    S = self.g.create_mesh(range(3, 3 + extra))
    answer = run_baker(self.X, self.R, S)
    lin_err   = abs(self.exact - answer['qlin'])
    final_err = abs(self.exact - answer['final'])
    self.assertTrue(lin_err >= final_err)
  def test_RunBaker_5_extra_point(self, extra=5):
    S = self.g.create_mesh(range(3, 3 + extra))
    answer = run_baker(self.X, self.R, S)
    lin_err   = abs(self.exact - answer['qlin'])
    final_err = abs(self.exact - answer['final'])
    self.assertTrue(lin_err >= final_err)

if __name__ == '__main__':
  suite = unittest.TestLoader().loadTestsFromTestCase(TestSequenceFunctions)
  unittest.TextTestRunner(verbosity=3).run(suite)
